@@ -20,71 +20,28 @@ from absl.testing import parameterized
 
 import chex
 import jax
-from jax.experimental import optimizers
 import jax.numpy as jnp
 
 from optax._src import alias
-from optax._src import transform
 from optax._src import update
 
 
-STEPS = 50
-LR = 1e-2
-
-
 class AliasTest(chex.TestCase):
 
-  def setUp(self):
-    super(AliasTest, self).setUp()
-    self.init_params = (jnp.array([1., 2.]), jnp.array([3., 4.]))
-    self.per_step_updates = (jnp.array([500., 5.]), jnp.array([300., 3.]))
-
-  @chex.all_variants()
   @parameterized.named_parameters(
-      ('sgd', alias.sgd(LR, 0.0),
-       optimizers.sgd(LR), 1e-5),
-      ('adam', alias.adam(LR, 0.9, 0.999, 1e-8),
-       optimizers.adam(LR, 0.9, 0.999), 1e-4),
-      ('rmsprop', alias.rmsprop(LR, .9, 0.1),
-       optimizers.rmsprop(LR, .9, 0.1), 1e-5),
-      ('adagrad', alias.adagrad(LR, 0., 0.,),
-       optimizers.adagrad(LR, 0.), 1e-5),
-  )
-  def test_jax_optimizer_equivalent(self, optax_optimizer, jax_optimizer, rtol):
-
-    # experimental/optimizers.py
-    jax_params = self.init_params
-    opt_init, opt_update, get_params = jax_optimizer
-    state = opt_init(jax_params)
-    for i in range(STEPS):
-      state = opt_update(i, self.per_step_updates, state)
-      jax_params = get_params(state)
-
-    # optax
-    optax_params = self.init_params
-    state = optax_optimizer.init(optax_params)
-
-    @self.variant
-    def step(updates, state):
-      return optax_optimizer.update(updates, state)
-
-    for _ in range(STEPS):
-      updates, state = step(self.per_step_updates, state)
-      optax_params = update.apply_updates(optax_params, updates)
-
-    # Check equivalence.
-    chex.assert_tree_all_close(jax_params, optax_params, rtol=rtol)
-
-  @parameterized.named_parameters(
-      ('sgd', alias.sgd(1e-2, 0.0)),
-      ('adam', alias.adam(1e-1)),
-      ('adamw', alias.adamw(1e-1)),
-      ('lamb', alias.adamw(1e-1)),
-      ('rmsprop', alias.rmsprop(1e-1)),
-      ('fromage', transform.scale_by_fromage(-1e-2)),
-      ('adabelief', alias.adabelief(1e-1)),
+      ('sgd', lambda: alias.sgd(1e-2, 0.0)),
+      ('adam', lambda: alias.adam(1e-1)),
+      ('adamw', lambda: alias.adamw(1e-1)),
+      ('lamb', lambda: alias.adamw(1e-1)),
+      ('rmsprop', lambda: alias.rmsprop(1e-1)),
+      ('fromage', lambda: alias.fromage(1e-2)),
+      ('adabelief', lambda: alias.adabelief(1e-1)),
+      ('radam', lambda: alias.radam(1e-1)),
+      ('yogi', lambda: alias.yogi(1.0)),
   )
   def test_parabel(self, opt):
+    opt = opt()
+
     initial_params = jnp.array([-1.0, 10.0, 1.0])
     final_params = jnp.array([1.0, -1.0, 1.0])
 
@@ -106,15 +63,19 @@ class AliasTest(chex.TestCase):
     chex.assert_tree_all_close(params, final_params, rtol=1e-2, atol=1e-2)
 
   @parameterized.named_parameters(
-      ('sgd', alias.sgd(2e-3, 0.2)),
-      ('adam', alias.adam(1e-1)),
-      ('adamw', alias.adamw(1e-1)),
-      ('lamb', alias.adamw(1e-1)),
-      ('rmsprop', alias.rmsprop(5e-3)),
-      ('fromage', transform.scale_by_fromage(-5e-3)),
-      ('adabelief', alias.adabelief(1e-1)),
+      ('sgd', lambda: alias.sgd(2e-3, 0.2)),
+      ('adam', lambda: alias.adam(1e-1)),
+      ('adamw', lambda: alias.adamw(1e-1)),
+      ('lamb', lambda: alias.adamw(1e-1)),
+      ('rmsprop', lambda: alias.rmsprop(5e-3)),
+      ('fromage', lambda: alias.fromage(5e-3)),
+      ('adabelief', lambda: alias.adabelief(1e-1)),
+      ('radam', lambda: alias.radam(1e-3)),
+      ('yogi', lambda: alias.yogi(1.0)),
   )
   def test_rosenbrock(self, opt):
+    opt = opt()
+
     a = 1.0
     b = 100.0
     initial_params = jnp.array([0.0, 0.0])
