@@ -19,6 +19,7 @@ from typing import Any, Callable, NamedTuple, Optional, Sequence, Tuple, Union
 import chex
 import jax
 import jax.numpy as jnp
+from optax._src import schedule
 
 
 # pylint:disable=no-value-for-parameter
@@ -26,17 +27,22 @@ OptState = NamedTuple  # Transformation states are (possibly empty) namedtuples.
 Params = Any  # Parameters are arbitrary nests of `jnp.ndarrays`.
 Updates = Params  # Gradient updates are of the same type as parameters.
 
+# Function used to initialise the transformation's state.
+TransformInitFn = Callable[
+    [Params],
+    Union[OptState, Sequence[OptState]]]
+# Function used to apply a transformation.
+TransformUpdateFn = Callable[
+    [Updates, OptState, Optional[Params]],
+    Tuple[Updates, OptState]]
+
 
 class GradientTransformation(NamedTuple):
   """Optax transformations consists of a function pair: (initialise, update)."""
-  init: Callable[  # Function used to initialise the transformation's state.
-      [Params], Union[OptState, Sequence[OptState]]]
-  update: Callable[  # Function used to apply a transformation.
-      [Updates, OptState, Optional[Params]], Tuple[Updates, OptState]]
+  init: TransformInitFn
+  update: TransformUpdateFn
 
 
-InitUpdate = GradientTransformation
-
 NO_PARAMS_MSG = (
     'You are using a transformation that requires the current value of'
     ' parameters, but you are not passing `params` when calling `update`.')
@@ -129,6 +135,64 @@ def clip_by_global_norm(max_norm) -> GradientTransformation:
   return GradientTransformation(init_fn, update_fn)
 
 
+def unitwise_norm(x):
+  """Computes norms of each output unit separately."""
+  if len(jnp.squeeze(x).shape) <= 1:  # Scalars and vectors
+    axis = None
+    keepdims = False
+  # Note that this assumes parameters with a shape of length 3 are multihead
+  # linear parameters--if you wish to apply AGC to 1D convs, you may need
+  # to modify this line.
+  elif len(x.shape) in [2, 3]:  # Linear layers of shape IO or multihead linear
+    axis = 0
+    keepdims = True
+  elif len(x.shape) == 4:  # Conv kernels of shape HWIO
+    axis = [0, 1, 2,]
+    keepdims = True
+  else:
+    raise ValueError(f'Got a parameter with shape not in [1, 2, 3, 4]! {x}')
+  return jnp.sum(x ** 2, axis=axis, keepdims=keepdims) ** 0.5
+
+
+def unitwise_clip(g_norm, max_norm, grad):
+  """Applies gradient clipping unit-wise."""
+  trigger = g_norm < max_norm
+  # This little max(., 1e-6) is distinct from the normal eps and just prevents
+  # division by zero. It technically should be impossible to engage.
+  clipped_grad = grad * (max_norm / jnp.maximum(g_norm, 1e-6))
+  return jnp.where(trigger, grad, clipped_grad)
+
+
+def adaptive_grad_clip(clipping, eps=1e-3) -> GradientTransformation:
+  """Clip updates to be at most clipping * parameter_norm, unit-wise.
+
+  References:
+    [Brock, Smith, De, Simonyan 2021] High-Performance Large-Scale Image
+    Recognition Without Normalization. (https://arxiv.org/abs/2102.06171)
+
+  Args:
+    clipping: Maximum allowed ratio of update norm to parameter norm.
+    eps: epsilon term to prevent clipping of zero-initialized params.
+
+  Returns:
+    An (init_fn, update_fn) tuple.
+  """
+
+  def init_fn(_):
+    return ClipByGlobalNormState()
+
+  def update_fn(updates, state, params):
+    g_norm = jax.tree_map(unitwise_norm, updates)
+    p_norm = jax.tree_map(unitwise_norm, params)
+    # Maximum allowable norm
+    max_norm = jax.tree_map(lambda x: clipping * jnp.maximum(x, eps), p_norm)
+    # If grad norm > clipping * param_norm, rescale
+    updates = jax.tree_multimap(unitwise_clip, g_norm, max_norm, updates)
+    return updates, state
+
+  return GradientTransformation(init_fn, update_fn)
+
+
 class TraceState(OptState):
   """Holds an aggregation of past updates."""
   trace: Params
@@ -415,11 +479,109 @@ def scale_by_belief(
   return GradientTransformation(init_fn, update_fn)
 
 
-class AdditiveWeightDecayState(NamedTuple):
+def scale_by_yogi(
+    b1: float = 0.9, b2: float = 0.999,
+    eps: float = 1e-3, eps_root: float = 0.0,
+    initial_accumulator_value: float = 1e-6) -> GradientTransformation:
+  """Rescale updates according to the Adam algorithm.
+
+  References:
+    [Zaheer et al, 2018](https://papers.nips.cc/paper/2018/hash/90365351ccc7437a1309dc64e4db32a3-Abstract.html) #pylint:disable=line-too-long
+
+  Args:
+    b1: decay rate for the exponentially weighted average of grads.
+    b2: decay rate for the exponentially weighted average of variance of grads.
+    eps: term added to the denominator to improve numerical stability.
+    eps_root: term added to the denominator inside the square-root to improve
+      numerical stability when backpropagating gradients through the rescaling.
+    initial_accumulator_value: The starting value for accumulators.
+      Only positive values are allowed.
+
+  Returns:
+    An (init_fn, update_fn) tuple.
+  """
+
+  def init_fn(params):
+    value_like = lambda p: jnp.full_like(p, initial_accumulator_value)
+    mu = jax.tree_map(value_like, params)  # First moment
+    nu = jax.tree_map(value_like, params)  # Second Central moment
+    return ScaleByAdamState(count=jnp.zeros([], jnp.int32), mu=mu, nu=nu)
+
+  def update_fn(updates, state, params=None):
+    del params
+    mu = _update_moment(updates, state.mu, b1, 1)
+    signed_sq = jax.tree_multimap(
+        lambda g, v: jnp.sign(v - g**2)*g**2, updates, state.nu)
+    nu = _update_moment(signed_sq, state.nu, b2, 2)
+    count_inc = _safe_int32_increment(state.count)
+    mu_hat = _bias_correction(mu, b1, count_inc)
+    nu_hat = _bias_correction(nu, b2, count_inc)
+    updates = jax.tree_multimap(
+        lambda m, v: m / (jnp.sqrt(v + eps_root) + eps), mu_hat, nu_hat)
+    return updates, ScaleByAdamState(count=count_inc, mu=mu, nu=nu)
+
+  return GradientTransformation(init_fn, update_fn)
+
+
+def scale_by_radam(b1: float = 0.9,
+                   b2: float = 0.999,
+                   eps: float = 1e-8,
+                   eps_root: float = 0.0,
+                   threshold: float = 5.0) -> GradientTransformation:
+  """Rescale updates according to the Rectified Adam algorithm.
+
+  References:
+    [Liu et al, 2020](https://arxiv.org/abs/1908.03265)
+
+  Args:
+    b1: decay rate for the exponentially weighted average of grads.
+    b2: decay rate for the exponentially weighted average of squared grads.
+    eps: term added to the denominator to improve numerical stability.
+    eps_root: term added to the denominator inside the square-root to improve
+      numerical stability when backpropagating gradients through the rescaling.
+    threshold: Threshold for variance tractability
+
+  Returns:
+    An (init_fn, update_fn) tuple.
+  """
+
+  ro_inf = 2./(1 - b2) - 1
+  def _radam_update(params):
+    ro = params[0]
+    mu_hat = params[1]
+    nu_hat = params[2]
+    r = jnp.sqrt((ro - 4)*(ro - 2)*ro_inf/((ro_inf - 4)*(ro_inf - 2)*ro))
+    updates = jax.tree_multimap(
+        lambda m, v: r*m / (jnp.sqrt(v + eps_root) + eps), mu_hat, nu_hat)
+    return updates
+
+  def init_fn(params):
+    mu = jax.tree_map(jnp.zeros_like, params)  # First moment
+    nu = jax.tree_map(jnp.zeros_like, params)  # Second moment
+    return ScaleByAdamState(count=jnp.zeros([], jnp.int32), mu=mu, nu=nu)
+
+  def update_fn(updates, state, params=None):
+    del params
+    mu = _update_moment(updates, state.mu, b1, 1)
+    nu = _update_moment(updates, state.nu, b2, 2)
+    count_inc = _safe_int32_increment(state.count)
+    b2t = b2**count_inc
+    ro = ro_inf - 2 * count_inc * b2t / (1 - b2t)
+    mu_hat = _bias_correction(mu, b1, count_inc)
+    nu_hat = _bias_correction(nu, b2, count_inc)
+    updates = jax.lax.cond(
+        ro >= threshold, _radam_update, lambda _: mu_hat,
+        (ro, mu_hat, nu_hat))
+    return updates, ScaleByAdamState(count=count_inc, mu=mu, nu=nu)
+
+  return GradientTransformation(init_fn, update_fn)
+
+
+class AddDecayedWeightsState(NamedTuple):
   """The decay transformation is stateless."""
 
 
-def additive_weight_decay(weight_decay: float = 0.0) -> GradientTransformation:
+def add_decayed_weights(weight_decay: float = 0.0) -> GradientTransformation:
   """Add parameter scaled by `weight_decay`.
 
   Args:
@@ -430,9 +592,11 @@ def additive_weight_decay(weight_decay: float = 0.0) -> GradientTransformation:
   """
 
   def init_fn(_):
-    return AdditiveWeightDecayState()
+    return AddDecayedWeightsState()
 
   def update_fn(updates, state, params):
+    if params is None:
+      raise ValueError(NO_PARAMS_MSG)
     updates = jax.tree_multimap(
         lambda g, p: g + weight_decay * p, updates, params)
     return updates, state
@@ -440,12 +604,17 @@ def additive_weight_decay(weight_decay: float = 0.0) -> GradientTransformation:
   return GradientTransformation(init_fn, update_fn)
 
 
+# TODO(b/180608630): Remove deprecated references.
+AdditiveWeightDecayState = AddDecayedWeightsState
+additive_weight_decay = add_decayed_weights
+
+
 class ScaleByScheduleState(OptState):
   """Maintains count for scale scheduling."""
   count: jnp.ndarray  # shape=(), dtype=jnp.int32
 
 
-def scale_by_schedule(step_size_fn: Callable[[jnp.ndarray], jnp.ndarray]):
+def scale_by_schedule(step_size_fn: schedule.Schedule):
   """Scale updates using a custom schedule for the `step_size`.
 
   Args:
@@ -493,65 +662,19 @@ def _safe_norm(x, min_norm):
   return jnp.where(norm < min_norm, min_norm, jnp.linalg.norm(x))
 
 
-def scale_by_fromage(
-    step_size: float = 1e-3,
-    min_norm: float = 1e-6,
-    step_size_factor_fn: Optional[Callable[[jnp.ndarray], jnp.ndarray]] = None,
-    ) -> GradientTransformation:
-  """Scale updates by Frobenius norm of parameters and grads.
-
-  References:
-    [Bernstein et. al 2020](https://arxiv.org/abs/2002.03432)
-
-  Args:
-    step_size: the step_size to multiply the updates by.
-    min_norm: Minimum norm of parameters and gradients, to avoid division by
-      zero and to ensure an update is made to a parameter with zero norm.
-    step_size_factor_fn: a function that takes an update count as input and
-      returns a factor the step_size is multiplied by.
-      Effective learning rate is step_size * step_size_factor_fn(state.count)
-
-  Returns:
-    An (init_fn, update_fn) tuple.
-  """
-
-  def init_fn(_: Params):
-    return ScaleByFromageState(count=jnp.zeros([], jnp.int32))
-
-  def update_fn(updates, state, params):
-    if params is None:
-      raise ValueError(NO_PARAMS_MSG)
-    lr = step_size
-    if step_size_factor_fn:
-      lr *= step_size_factor_fn(state.count)
-
-    def _scale_update(update, param):
-      update_norm = _safe_norm(update, min_norm)
-      param_norm = _safe_norm(param, min_norm)
-      norm_ratio = param_norm / update_norm
-      mult = jax.lax.rsqrt(1 + lr**2).astype(update.dtype)
-      scaled_update = lr * update * norm_ratio * mult
-      scaled_param_correction = param * (mult - 1.)
-      return scaled_update + scaled_param_correction
-
-    count_inc = _safe_int32_increment(state.count)
-    updates = jax.tree_multimap(_scale_update, updates, params)
-
-    return updates, ScaleByFromageState(count=count_inc)
-
-  return GradientTransformation(init_fn, update_fn)
-
-
 class ScaleByTrustRatioState(NamedTuple):
   """The scale and decay trust ratio transformation is stateless."""
 
 
-def scale_by_trust_ratio() -> GradientTransformation:
+def scale_by_trust_ratio(min_norm: float = 0.0) -> GradientTransformation:
   """Scale updates by trust ratio`.
 
   References:
     [You et. al 2020](https://arxiv.org/abs/1904.00962)
 
+  Args:
+    min_norm: minimum norm for params and gradient norms; by default is zero.
+
   Returns:
     An (init_fn, update_fn) tuple.
   """
@@ -564,10 +687,13 @@ def scale_by_trust_ratio() -> GradientTransformation:
       raise ValueError(NO_PARAMS_MSG)
 
     def _scale_update(update, param):
-      param_norm = jnp.linalg.norm(param)
-      update_norm = jnp.linalg.norm(update)
+
+      # Clip norms to minimum value, by default no clipping.
+      param_norm = _safe_norm(param, min_norm)
+      update_norm = _safe_norm(update, min_norm)
       trust_ratio = param_norm / update_norm
 
+      # If no minimum norm clipping is used
       # Set trust_ratio to 1 in case where parameters would never be updated.
       zero_norm = jnp.logical_or(param_norm == 0., update_norm == 0.)
       safe_trust_ratio = jnp.where(
@@ -633,6 +759,12 @@ class ApplyEvery(OptState):
 def apply_every(k: int = 1) -> GradientTransformation:
   """Accumulate gradients and apply them every k steps.
 
+  Note that if this transformation is part of a chain, the states of the other
+  transformations will still be updated at every step. In particular, using
+  `apply_every` with a batch size of N/2 and k=2 is not necessarily equivalent
+  to not using `apply_every` with a batch size of N. If this equivalence is
+  important for you, consider using the `optax.MultiSteps`.
+
   Args:
     k: emit non-zero gradients every k steps, otherwise accumulate them.
 
@@ -688,3 +820,76 @@ def centralize() -> GradientTransformation:
     return updates, state
 
   return GradientTransformation(init_fn, update_fn)
+
+
+class NonNegativeParamsState(OptState):
+  """The `keep_params_nonnegative` transformation is stateless."""
+
+
+def keep_params_nonnegative() -> GradientTransformation:
+  """Modifies the updates to keep parameters non-negative, i.e. >= 0.
+
+  This transformation ensures that parameters after the update will be
+  larger than or equal to zero.
+  In a chain of transformations, this should be the last one.
+
+  WARNING: the transformation expects input params to be non-negative.
+  When params is negative the transformed update will move them to 0.
+
+  Returns:
+    An (init_fn, update_fn) tuple.
+  """
+
+  def init_fn(_):
+    return NonNegativeParamsState()
+
+  def update_fn(updates, state, params):
+    if params is None:
+      raise ValueError(NO_PARAMS_MSG)
+
+    updates = jax.tree_multimap(
+        lambda p, u: jnp.where((p + u) < 0., -p, u), params, updates)
+    return updates, state
+
+  return GradientTransformation(init_fn, update_fn)
+
+
+class ZeroNansState(OptState):
+  """Contains a tree.
+
+  The entry `found_nan` has the same tree structure as that of the parameters.
+  Each leaf is a single boolean which contains True iff a NaN was detected in
+  the corresponding parameter array at the last call to `update`.
+  """
+  found_nan: Any
+
+
+def zero_nans() -> GradientTransformation:
+  """A transformation which replaces NaNs with 0.
+
+  Zeroing values in gradients is guaranteed to produce a direction of
+  non-increasing loss.
+
+  The state of the transformation has the same tree structure as that of the
+  parameters. Each leaf is a single boolean which contains True iff a NaN was
+  detected in the corresponding parameter array at the last call to `update`.
+  This state is not used by the transformation internally, but lets users be
+  aware when NaNs have been zeroed out.
+
+  Returns:
+    A `GradientTransformation`.
+  """
+
+  def init_fn(params):
+    return ZeroNansState(
+        jax.tree_map(lambda p: jnp.array(False, dtype=jnp.bool_), params))
+
+  def update_fn(updates, opt_state, params=None):
+    del params
+    opt_state = ZeroNansState(
+        jax.tree_map(lambda p: jnp.any(jnp.isnan(p)), updates))
+    updates = jax.tree_map(
+        lambda p: jnp.where(jnp.isnan(p), jnp.zeros_like(p), p), updates)
+    return updates, opt_state
+
+  return GradientTransformation(init=init_fn, update=update_fn)
