@@ -17,9 +17,11 @@
 
 from absl.testing import absltest
 from absl.testing import parameterized
+
 import chex
 import jax
 import jax.numpy as jnp
+
 from optax._src import alias
 from optax._src import combine
 from optax._src import transform
@@ -40,7 +42,6 @@ class TransformTest(parameterized.TestCase):
   @parameterized.named_parameters([
       ('adam', transform.scale_by_adam),
       ('rmsprop', transform.scale_by_rms),
-      ('fromage', transform.scale_by_fromage),
       ('stddev', transform.scale_by_stddev),
       ('trust_ratio', transform.scale_by_trust_ratio),
   ])
@@ -59,23 +60,6 @@ class TransformTest(parameterized.TestCase):
     jax.tree_multimap(lambda *args: chex.assert_equal_shape(args), params,
                       updates)
 
-  @chex.all_variants()
-  def test_scale_by_fromage(self):
-    schedule = lambda c: 1.0/(c + 1.0)
-    fromage = transform.scale_by_fromage(step_size_factor_fn=schedule)
-    params = self.init_params
-    state = fromage.init(params)
-    previous_norm = jnp.inf
-    transform_fn = self.variant(fromage.update)
-
-    for _ in range(STEPS):
-      # Apply a step of fromage
-      updates, state = transform_fn(self.per_step_updates, state, params)
-      # Updates should get smaller due to the the learning schedule.
-      norm = transform.global_norm(updates)
-      self.assertLess(norm, previous_norm)
-      previous_norm = norm
-
   @chex.all_variants()
   def test_apply_every(self):
     # The frequency of the application of sgd
@@ -168,5 +152,85 @@ class TransformTest(parameterized.TestCase):
     centralized_inputs, _ = centralizer.update(inputs, None)
     chex.assert_tree_all_close(centralized_inputs, outputs)
 
+  def test_keep_params_nonnegative(self):
+    grads = (jnp.array([500., -500., 0.]),
+             jnp.array([500., -500., 0.]),
+             jnp.array([500., -500., 0.]))
+
+    params = (jnp.array([-1., -1., -1.]),
+              jnp.array([1., 1., 1.]),
+              jnp.array([0., 0., 0.]))
+
+    # vanilla sgd
+    opt = combine.chain(
+        transform.trace(decay=0, nesterov=False), transform.scale(-LR))
+    opt_state = opt.init(params)
+
+    updates, _ = opt.update(grads, opt_state, params)
+    new_params = update.apply_updates(params, updates)
+
+    chex.assert_tree_all_close(new_params, (jnp.array([-6., 4., -1.]),
+                                            jnp.array([-4., 6., 1.]),
+                                            jnp.array([-5., 5., 0.])))
+
+    # sgd with keeping parameters non-negative
+    opt = combine.chain(
+        transform.trace(decay=0, nesterov=False), transform.scale(-LR),
+        transform.keep_params_nonnegative())
+    opt_state = opt.init(params)
+
+    updates, _ = opt.update(grads, opt_state, params)
+    new_params = update.apply_updates(params, updates)
+
+    chex.assert_tree_all_close(new_params, (jnp.array([0., 4., 0.]),
+                                            jnp.array([0., 6., 1.]),
+                                            jnp.array([0., 5., 0.])))
+
+  @chex.all_variants
+  def test_zero_nans(self):
+    params = (jnp.zeros([3]), jnp.zeros([3]), jnp.zeros([3]))
+
+    opt = transform.zero_nans()
+    opt_state = self.variant(opt.init)(params)
+    update_fn = self.variant(opt.update)
+
+    equality_comp = lambda a, b: bool(jnp.all(jnp.equal(a, b)))
+    chex.assert_tree_all_equal_comparator(equality_comp, opt_state,
+                                          (jnp.array(False),) * 3)
+
+    # Check an upate with nans
+    grads_with_nans = (jnp.ones([3]),
+                       jnp.array([1., float('nan'), float('nan')]),
+                       jnp.array([float('nan'), 1., 1.]))
+    updates, opt_state = update_fn(grads_with_nans, opt_state)
+    chex.assert_tree_all_equal_comparator(
+        equality_comp, opt_state,
+        (jnp.array(False), jnp.array(True), jnp.array(True)))
+    chex.assert_tree_all_equal_comparator(
+        equality_comp, updates,
+        (jnp.ones([3]), jnp.array([1., 0., 0.]), jnp.array([0., 1., 1.])))
+
+    # Check an upate with nans and infs
+    grads_with_nans_infs = (jnp.ones([3]),
+                            jnp.array([1., float('nan'), float('nan')]),
+                            jnp.array([float('inf'), 1., 1.]))
+    updates, opt_state = update_fn(grads_with_nans_infs, opt_state)
+    chex.assert_tree_all_equal_comparator(
+        equality_comp, opt_state,
+        (jnp.array(False), jnp.array(True), jnp.array(False)))
+    chex.assert_tree_all_equal_comparator(
+        equality_comp, updates,
+        (jnp.ones([3]), jnp.array([1., 0., 0.]),
+         jnp.array([float('inf'), 1., 1.])))
+
+    # Check an upate with only good values
+    grads = (jnp.ones([3]), jnp.ones([3]), jnp.ones([3]))
+    updates, opt_state = update_fn(grads, opt_state)
+    chex.assert_tree_all_equal_comparator(
+        equality_comp, opt_state,
+        (jnp.array(False), jnp.array(False), jnp.array(False)))
+    chex.assert_tree_all_equal_comparator(equality_comp, updates, grads)
+
+
 if __name__ == '__main__':
   absltest.main()
